@@ -28,6 +28,7 @@ use Autodia::Diagram::Superclass;
 use Autodia::Diagram::Dependancy;
 use Autodia::Diagram::Inheritance;
 use Autodia::Diagram::Relation;
+use Autodia::Diagram::Realization;
 
 my %dot_filetypes = (
 		     gif => 'as_gif',
@@ -133,6 +134,16 @@ sub add_dependancy
     return 1;
 }
 
+sub add_realization {
+   my $self        = shift;
+   my $realization = shift;
+
+   $self->_package_add($realization);
+   $realization->Set_Id( $self->_object_count );
+
+   return 1;
+ }
+
 sub add_inheritance {
     my $self = shift;
     my $inheritance = shift;
@@ -417,6 +428,26 @@ sub Relations {
     return \@relations;
   }
 
+sub Realizations {
+   my $self = shift;
+
+   unless( defined $self->{"packages"}{"realization"} ) {
+     print STDERR "Realizations Diagram.pm : none to be printed - ignoring..\n
+";
+     return 0;
+   }
+
+   my @realizations;
+   my %realizations = %{ $self->{"packages"}{"realization"} };
+   my @keys         = keys %realizations;
+   my $i            = 0;
+
+   foreach my $key (@keys) {
+     $realizations[ $i++ ] = $realizations{$key};
+   }
+
+   return \@realizations;
+ }
 
 sub Dependancies
   {
@@ -478,9 +509,9 @@ sub export_graphviz
 	    next if ($method->{visibility} == 1 && $config{public});
 	    my $method_string = ($method->{visibility} == 0) ? '+ ' : '- ';
 	    $method_string .= $method->{name}."(";
-	    if (ref $method->{"Param"} ) {
+	    if (ref $method->{"Params"} ) {
 	      my @args = ();
-	      foreach my $argument ( @{$method->{"Param"}} ) {
+	      foreach my $argument ( @{$method->{"Params"}} ) {
 		  push (@args, ((defined ($argument->{Type}) )? $argument->{Type} . " " . $argument->{Name} : $argument->{Name}));
 	      }
 	      $method_string .= join (", ",@args) if (scalar @args);
@@ -625,9 +656,9 @@ sub export_springgraph
 	    next if ($method->{visibility} == 1 && $config{public});
 	    my $method_string = ($method->{visibility} == 0) ? '+ ' : '- ';
 	    $method_string .= $method->{name}."(";
-	    if (ref $method->{"Param"} ) {
+	    if (ref $method->{"Params"} ) {
 	      my @args = ();
-	      foreach my $argument ( @{$method->{"Param"}} ) {
+	      foreach my $argument ( @{$method->{"Params"}} ) {
 		  push (@args, ((defined ($argument->{Type}) )? $argument->{Type} . " " . $argument->{Name} : $argument->{Name}));
 	      }
 	      $method_string .= join (", ",@args) if (scalar @args);
@@ -753,9 +784,9 @@ sub export_vcg {
 	  next if ($method->{visibility} == 1 && $config{public});
 	  my $method_string = ($method->{visibility} == 0) ? '+ ' : '- ';
 	  $method_string .= $method->{name}."(";
-	  if (ref $method->{"Param"} ) {
+	  if (ref $method->{"Params"} ) {
 	    my @args = ();
-	    foreach my $argument ( @{$method->{"Param"}} ) {
+	    foreach my $argument ( @{$method->{"Params"}} ) {
 	      push (@args, $argument->{Type} . " " . $argument->{Name});
 	    }
 	    $method_string .= join (", ",@args);
@@ -882,8 +913,8 @@ sub export_xml
  	my ($methods) = ($Class->Operations);
 	foreach my $method (@$methods) {
 	  $method->{name}=xml_escape($method->{name});
-	  if (ref $method->{"Param"} ) {
-	    foreach my $argument ( @{$method->{"Param"}} ) {
+	  if (ref $method->{"Params"} ) {
+	    foreach my $argument ( @{$method->{"Params"}} ) {
 		$argument->{Type} = xml_escape($argument->{Type}) if (defined $argument->{Type});
 		$argument->{Name} = xml_escape($argument->{Name});
 		$argument->{Kind} = xml_escape($argument->{Kind}) if (defined $argument->{Kind});
@@ -1139,6 +1170,16 @@ sub _layout_dia_new {
     }
   }
 
+   # add realization edges
+   my $realizations = $self->Realizations;
+   if( ref $realizations ) {
+     foreach my $Realization (@$realizations) {
+       push( @edges,
+         { to => $Realization->Child, from => $Realization->Parent } );
+     }
+   }
+
+
   # add relation edges
   my $relations = $self->Relations;
   if (ref $relations) {
@@ -1299,6 +1340,9 @@ sub _layout_dia_new {
   if (ref $self->Dependancies)
     { push(@relationships, @{$self->Dependancies}); }
 
+  if( ref $self->Realizations ) {
+   push( @relationships, @{ $self->Realizations } );}
+
   if (ref $self->Inheritances)
     { push(@relationships, @{$self->Inheritances}); }
 
@@ -1633,6 +1677,9 @@ my @relationships = ();
     if (ref $self->Dependancies)
       {	push(@relationships, @{$self->Dependancies}); }
 
+    if( ref $self->Realizations ) {
+     push( @relationships, @{ $self->Realizations } );}
+
     if (ref $self->Inheritances)
       { push(@relationships, @{$self->Inheritances}); }
 
@@ -1709,7 +1756,7 @@ sub get_umbrello_template {
     [% FOREACH op = class.Operations %]
     <UML:Operation isSpecification="false" isLeaf="false" xmi.id="[% op.Id %]" type="[% op.type | html  %]" isRoot="false" isAbstract="false" name="[% op.name | html  %]" >
      <UML:BehavioralFeature.parameter>
-      [% FOREACH par = op.Param %]
+      [% FOREACH par = op.Params %]
       <UML:Parameter isSpecification="false" isLeaf="false" visibility="private" xmi.id="[% par.Id %]" isRoot="false" value="[% par.value %]" type="[% par.type | html  %]" isAbstract="false" name="[% par.name | html %]" />
        [% END %]
      </UML:BehavioralFeature.parameter>
@@ -1734,7 +1781,7 @@ documentation="" name="[% at.name FILTER html %]" static="0" scope="200" />
       [% FOREACH op = class.Operations %]
       [% xmictr = xmictr + 1 %]
       <UML:Operation stereotype="" package="" xmi.id="[% xmictr %]" type="[% op.type %]" abstract="0" documentation="" name="[% op.name %]" static="0" scope="200" >
-         [% FOREACH par = op.Param %]
+         [% FOREACH par = op.Params %]
          [% xmictr = xmictr + 1 %]
          <UML:Parameter stereotype="" package="" xmi.id="[% xmictr %]" value="[% par.value %]" type="[% par.type  FILTER html %]" abstract="0" documentation="" name="[% par.Name FILTER html %]" static="0" scope="200" />
          [% END %]
@@ -1999,9 +2046,9 @@ sub get_default_template {
           <dia:attribute name="class_scope">
             <dia:boolean val="false"/>
           </dia:attribute>
-	  [% IF op.Param.0 %]
+	  [% IF op.Params.0 %]
           <dia:attribute name="parameters">
-            [% FOREACH par = op.Param %] 
+            [% FOREACH par = op.Params %] 
             <dia:composite type="umlparameter">
               <dia:attribute name="name">
                 <dia:string>#[% par.Name FILTER html %]#</dia:string>
@@ -2087,6 +2134,44 @@ sub get_default_template {
  </dia:object>
 [% END %]
 [% # %]
+[% SET realizations = diagram.Realizations %]
+[% # %]
+[% FOREACH realization = realizations %]
+ <dia:object type="UML - Realizes" version="0" id="O[% realization.Id %]">
+   <dia:attribute name="obj_pos">
+     <dia:point val="[% realization.Orth_Top_Right %]"/>
+   </dia:attribute>
+   <dia:attribute name="obj_bb">
+     <dia:rectangle val="[% realization.Orth_Top_Right %];[% realization.Orth
+_Bottom_Left %]"/>
+   </dia:attribute>
+   <dia:attribute name="orth_points">
+     <dia:point val="[% realization.Orth_Bottom_Left%]"/>
+     <dia:point val="[% realization.Orth_Mid_Left %]"/>
+     <dia:point val="[% realization.Orth_Mid_Right %]"/>
+     <dia:point val="[% realization.Orth_Top_Right%]"/>
+   </dia:attribute>
+   <dia:attribute name="orth_orient">
+     <dia:enum val="1"/>
+     <dia:enum val="0"/>
+     <dia:enum val="1"/>
+   </dia:attribute>
+   <dia:attribute name="draw_arrow">
+     <dia:boolean val="true"/>
+   </dia:attribute>
+   <dia:attribute name="name">
+     <dia:string/>
+   </dia:attribute>
+   <dia:attribute name="stereotype">
+     <dia:string/>
+   </dia:attribute>
+   <dia:connections>
+     <dia:connection handle="1" to="O[% realization.Child %]" connection="6"/>
+     <dia:connection handle="0" to="O[% realization.Parent %]" connection="1"/>
+   </dia:connections>
+ </dia:object>
+[% END %]
+[% # %]
 [% SET dependancies = diagram.Dependancies %]
 [% # %]
 [% FOREACH dependancy = dependancies %]
